package ch.unige.solidify.repository;

import java.util.List;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.data.repository.NoRepositoryBean;

import ch.unige.solidify.rest.FacetPage;
import ch.unige.solidify.rest.FacetRequest;
import ch.unige.solidify.rest.SearchCondition;

@NoRepositoryBean
public interface SolidifyRepositoryNoSql<T> {
  boolean delete(T t);

  // CRUD functions
  List<T> findAll(String indexName, T search);

  default Page<T> findAll(String indexName, T search, Pageable pageable) {
    final List<T> list = this.findAll(indexName, search);
    final long start = pageable.getOffset();
    final long end = (start + pageable.getPageSize()) > list.size() ? list.size()
            : (start + pageable.getPageSize());
    return new PageImpl<>(list.subList((int) start, (int) end), pageable, list.size());
  }

  T findOne(String indexName, String id);

  T save(T t);

  // Default Functions
  default Page<T> search(String indexName, String query, Pageable pageable) {
    return this.search(indexName, query, (String) null, null, null, null, pageable);
  }

  default FacetPage<T> search(String indexName, String query, String conditions, List<FacetRequest> facetRequests, Pageable pageable) {
    Page page = this.search(indexName, query, (String) null, null, null, null, pageable);

    if (page instanceof FacetPage) {
      return (FacetPage<T>) page;
    } else {
      return new FacetPage<T>(page.getContent(), null, page.getPageable(), page.getTotalElements());
    }
  }

  default Page<T> search(String indexName, String query, String set, String field,
          String from, String to, Pageable pageable) {
    return this.findAll(indexName, null, pageable);
  }

  default FacetPage<T> search(String indexName, String query, List<FacetRequest> facetRequests, String set, String field,
          String from, String to, String filter, Pageable pageable) {
    Page page = this.findAll(indexName, null, pageable);

    if (page instanceof FacetPage) {
      return (FacetPage<T>) page;
    } else {
      return new FacetPage<T>(page.getContent(), null, page.getPageable(), page.getTotalElements());
    }
  }

  default Page<T> searchRange(String indexName, String field, String from, String to,
          Pageable pageable) {
    return this.findAll(indexName, null, pageable);
  }

  default FacetPage<T> search(String indexName, List<SearchCondition> conditions, List<FacetRequest> facetRequests, Pageable pageable) {
    return this.search(indexName, null, null, facetRequests, pageable);
  }

  // Define Database name ~ index name
  void setIndex(String indexName);

  T update(T t);
}
