package ch.unige.solidify.test.controller;

import static org.hamcrest.CoreMatchers.is;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.OffsetDateTime;
import java.util.ArrayList;
import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MockMvc;

import ch.unige.solidify.TestTools;
import ch.unige.solidify.test.controller.config.MockApplication;
import ch.unige.solidify.test.controller.dao.MockUser;
import ch.unige.solidify.test.controller.dao.MockUserRepository;
import ch.unige.solidify.util.SearchCriteria;
import ch.unige.solidify.util.SearchOperation;

@RunWith(SpringRunner.class)
@AutoConfigureMockMvc
@SpringBootTest(classes = MockApplication.class)
@WithMockUser(username = "TEST", password = "TEST")
public class ResourceReadOnlyControllerTest {

  public static class ResourceReadOnlyControllerOracleDBTest {
  }

  @Autowired
  private MockMvc mockMvc;

  LocalDate baseDate = LocalDate.parse("2016-08-16");
  LocalDateTime baseDateTime = LocalDateTime.parse("2015-12-03T10:15:30");
  OffsetDateTime baseOffsetTime = OffsetDateTime.parse("2007-12-03T10:15:30+01:00");
  MockUser user1;
  MockUser user2;
  MockUser user3;

  @Autowired
  MockUserRepository userRepository;

  @Before
  public void setup() {
    this.user1 = new MockUser("1", "john", 22, new Integer("3"), new Long("2000"), new BigDecimal("200.58"),
            this.baseDate.minusYears(10), this.baseDateTime.minusYears(5).minusMonths(2), this.baseOffsetTime);
    this.user2 = new MockUser("2", "doe", 15, new Integer("1"), new Long("4000"), new BigDecimal("300.25"),
            this.baseDate.minusYears(20), this.baseDateTime.minusYears(10).minusMonths(2), this.baseOffsetTime);
    this.user3 = new MockUser("3", "jo", 40, new Integer("5"), new Long("3000"), new BigDecimal("400.13"),
            this.baseDate.minusYears(30), this.baseDateTime.minusYears(15).minusMonths(2), this.baseOffsetTime);

    this.userRepository.save(this.user1);
    this.userRepository.save(this.user2);
    this.userRepository.save(this.user3);
  }

  @Test
  public void testGetRequest() throws Exception {
    this.mockMvc.perform(get("/test?size=2&sort=age")).andExpect(status().isOk())
            .andExpect(content().contentType("application/hal+json"))
            .andExpect(jsonPath("_data[0].name", is("doe")));
  }

  @Test
  public void testGetSearch() throws Exception {
    this.mockMvc.perform(get("/test/search?search=i-name~o,age:15&match=all")).andExpect(status().isOk())
            .andExpect(content().contentType("application/hal+json"))
            // .andDo(print());
            .andExpect(jsonPath("_data[0].name", is("doe")));
  }

  @Test
  public void testGetSearchwithDate() throws Exception {
    this.mockMvc.perform(
            get("/test/search?search=meetingTime:2007-12-03T10:15:30+01:00,marriageTime∞2003-10-03T10:15:30_2016-02-03T12:15:30"))
            .andExpect(status().isOk())
            .andExpect(content().contentType("application/hal+json"))
            .andExpect(jsonPath("_data[0].name", is("john")));
  }

  @Test
  public void testGetWithId() throws Exception {
    this.mockMvc.perform(get("/test/3")).andExpect(status().isOk())
            .andExpect(content().contentType("application/hal+json"))
            .andExpect(jsonPath("name", is("jo")));
  }

  @Test
  public void testPostSearch() throws Exception {
    final List<SearchCriteria> allCriterias = new ArrayList<>();
    SearchCriteria criteria = new SearchCriteria("i", "name", SearchOperation.CONTAINS, "o");
    allCriterias.add(criteria);
    criteria = new SearchCriteria("c", "age", SearchOperation.LESS_THAN, "18");
    allCriterias.add(criteria);
    final MockUser user = new MockUser();
    user.setSearchCriterias(allCriterias);

    this.mockMvc.perform(post("/test/search?match=all").contentType(MediaType.APPLICATION_JSON)
            .content(TestTools.convertObjectToJsonBytes(user))).andExpect(status().isOk())
            .andExpect(content().contentType("application/hal+json"))
            .andExpect(jsonPath("_data[0].searchCriterias").doesNotExist())
            .andExpect(jsonPath("_data[0].name", is("doe")));
  }
}
