package ch.unige.solidify.converter;

import static java.util.Collections.emptyList;

import java.util.Arrays;
import java.util.List;

import javax.persistence.AttributeConverter;
import javax.persistence.Converter;

import ch.unige.solidify.util.StringTool;

@Converter
public class KeywordsConverter implements AttributeConverter<List<String>, String> {

  @Override
  public String convertToDatabaseColumn(List<String> keywords) {
    if (keywords == null || keywords.isEmpty()) {
      return null;
    }
    final StringBuilder keywordBuilder = new StringBuilder();
    for (final String keyword : keywords) {
      keywordBuilder.append(keyword);
      keywordBuilder.append(" ");
    }
    return keywordBuilder.toString();
  }

  @Override
  public List<String> convertToEntityAttribute(String keywords) {
    if (StringTool.isNullOrEmpty(keywords)) {
      return emptyList();
    }
    return Arrays.asList(keywords.split(" "));
  }
}
